#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

from django.utils import unittest
from models import Download
from api import DynamicLink

class Obj:
    pass

class DownloadTestCase(unittest.TestCase):
    """Test of the Doanload modell"""
    def setUp(self):
        self.path = '/static/public/testrunner/test.jpg'
        self.file = 'test.jpg'
        self.downl = Download.objects.create(
                              slug='download_model_unittest',
                              file_path=self.path,
                              timeout_hours = 10,
                              max_clicks = 10,
                              )

    def test_download_methodes(self):
        self.assertEqual(self.downl.get_filename(), 
                        self.file,
                        )
        self.assertFalse(self.downl.timeout_clicks())
        self.assertFalse(self.downl.timeout_time())
        self.assertFalse(self.downl.timeout())
        self.assertEqual(self.downl.get_path(), self.path)
    
class DynamicLinkTestCase(unittest.TestCase):
    """Test the DynamicLink API"""
    def setUp(self):
        self.path = '/static/public/testrunner/test.jpg'
        self.file = 'test.jpg'
        self.dlink = DynamicLink(
                      slug='api_unittest',
                      file_path=self.path,
                      timeout_hours = 10,
                      max_clicks = 10,
                      )

    def test_dynamic_link_methodes(self):
        request = Obj()
        request.__dict__['META']={}
        request.META['HTTP_HOST']='www.testrunner.eu'
        link = '%s/%s/%s/%s/%s' % ('http://www.testrunner.eu',
                                'lg',
                                'serve/link',
                                self.dlink.get_link_key(),
                                self.file,
                                )
        self.assertEqual(link, self.dlink.get_link_url(request))
        


#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

# This file holds the presettings of the app. 

# A path to a directory from witch walk down so you can choose your files.
DYNAMIC_LINK_MEDIA = getattr(settings, 'DYNAMIC_LINK_MEDIA', settings.MEDIA_ROOT)

# A string that modify the serve url path:
# /www.example.com/DYNAMIC_LINK_URL_BASE_COMPONENT/link/3839hd8HKl3/example.zip
DYNAMIC_LINK_URL_BASE_COMPONENT = getattr(settings, 'DYNAMIC_LINK_URL_BASE_COMPONENT', 'serve')

# It's here because of not violate the DRY priciple.
TEXT_REQUEST_DOES_NOT_EXIST = _(u'This request is faulty')
TEXT_REQUEST_IS_EXPIRED = _(u'Sorry, this request is already expired')


#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

__author__ = "Andreas Fritz - sources.e-blue.eu"
__copyright__ = "Copyright (c) " + "28.08.2010" + " Andreas Fritz"
__licence__ = "New BSD Licence"


import presettings
from django.db import models
import settings
from django.utils.translation import ugettext_lazy as _
import random
import os
import datetime

class Download(models.Model):
    slug = models.SlugField(verbose_name=_('slug'), blank=False, unique=True)
    active = models.BooleanField(default=True,
                                 verbose_name=_(u'is aktive'),
                                 )
    file_path = models.FilePathField(
                                      path=presettings.DYNAMIC_LINK_MEDIA,
                                      help_text=_(u"Select the content your like \
                                      to provide."),
                                      verbose_name=_(u'content to serve'),
                                      blank=True,
                                      recursive=True
                                      )
    link_key = models.CharField(max_length=50, editable=False, unique=True)
    timestamp_creation = models.DateTimeField(
                                              auto_now=False,
                                              auto_now_add=True,
                                              editable=False,
                                              verbose_name=_(u'creation time'),
                                              )
    timeout_hours = models.IntegerField(verbose_name=_(u'timout in hours'), \
                                        default=72, help_text=_('Zero value means no timeout.'))
    max_clicks = models.IntegerField(verbose_name=_(u'maximum allowed clicks'), \
                                     default=3, help_text=_('Zero value means no limitation.'))
    current_clicks = models.IntegerField(default=0, verbose_name=_(u'current clicks'))

    def get_filename(self):
        return os.path.basename(self.file_path)

    def timeout_clicks(self):
        """If max clicks reached than it returns True"""
        # dont cange the order
        if not self.max_clicks == 0: # max clicks 0 means never expired through clicks
            if self.current_clicks >= self.max_clicks: # if number of max clicks reached
                return True

    def timeout_time(self):
        """If timout time is reached it returs True"""
        if not self.timeout_hours == 0: # never timeout through time
            if self.timestamp_creation + datetime.timedelta(hours=self.timeout_hours) < datetime.datetime.now():
                return True

    def timeout(self):
        """
        In case of timeout (clicks or expired time) it returns True
        """
        if self.timeout_time():
            return True
        if self.timeout_clicks():
            return True

    def get_timout_time(self):
        """Is shown at the admit list display"""
        if self.timeout_hours == 0:
            return '<span style="color: #FF7F00; ">%s</span>' % unicode(_(u'never expires'))
        return (self.timestamp_creation + datetime.timedelta(hours=self.timeout_hours)) - datetime.datetime.now()

    def set_link(self, file, slug='autogenerated', timeout=None, maxclicks=None):
        self.slug = slug
        self.file_path = file.split(os.path.basename(settings.MEDIA_ROOT))[-1]
        if timeout:
            self.timeout_hours = timeout
        if maxclicks:
            self.max_clicks = maxclicks
        self.save()

    def __setup_instance(self):
        """
        Check if object instance expired and keep values of the istance actual.
        """
        # Keep the order of the tests
        
        # 1. Test of timeout
        if self.timeout():
            if self.active:
                self.active = False
                self.save()
                raise IsExpiredError()

        # 2. set max click values
        if self.current_clicks < self.max_clicks:
            self.current_clicks += 1
            # check of last cklick
            if self.current_clicks == self.max_clicks:
                self.active = False # if it was the last allowed cklick then set to expired
            self.save()
        elif self.max_clicks == 0:
            self.current_clicks += 1
            self.save()

    def get_path(self):
        """
        if active it returns the full path of stored file to serve.
        if expired it returns None
        """
        self.__setup_instance()
        return self.file_path

    def __gen_key(self):
        """
        function for generating random keys
        """
        #key = str(time.time()).replace('.', '')
        key = '' # for shorter keys
        characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890'
        key_length = 30
        for y in range(key_length):
            key += characters[random.randint(0, len(characters)-1)]
        return key

    def save(self, * args, ** kwargs):
        """Perform custom methodes before saving."""
        # If not avalible set unique link key bevor saving
        if not self.link_key:
            self.link_key = self.__gen_key()
        # call the real save method
        super(Download, self).save(*args, ** kwargs) # Call the "real" save() method

    def __unicode__(self):
        return '%s: %s, %s: %s' % (unicode(_(u'Slug')), self.slug, unicode(_(u'Filename')), self.get_filename())

class IsExpiredError(Exception):
    """Error class for expired link objects"""
    def __init__(self, value=''):
        self.value = presettings.TEXT_REQUEST_IS_EXPIRED + value
    def __str__(self):
        return repr(self.value)



#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

from models import Download
import presettings


def file_link_url(request, linkobject, langcode='lg'):
    """returns the access url of the of the dynamicLink object"""
    return '%s%s/%s/%s/link/%s/%s' % ('http://', request.META.get('HTTP_HOST'), \
            langcode, presettings.DYNAMIC_LINK_URL_BASE_COMPONENT, linkobject.link_key, \
            linkobject.get_filename())

class DynamicLink:
    """
    create and access dynamic links form outsite of this app
    """
    def __init__(self, slug, file_path, timeout_hours=None, max_clicks=None):
        """creates a new dynamic link"""
        self.new_link = Download()
        self.new_link.slug = slug
        self.new_link.file_path = file_path

        if timeout_hours:
            self.new_link.timeout_hours = timeout_hours
        if max_clicks:
            self.new_link.max_clicks = max_clicks

        self.new_link.save()

    def get_link_key(self):
        """get his unique key"""
        return self.new_link.link_key

    def get_link_url(self, request, langcode='lg'):
        """access his url"""
        return file_link_url(request, self.new_link, langcode)

class DownloadSiteUrl():
    """
    create a download site with links from the given keys
    """
    def __init__(self, keylist=None):
        """you can add a list with keys"""
        self.keys = []
        if keylist:
            self.keys += keylist

    def add_key(self,key):
        """add a key to allready given keys"""
        self.keys.append(key)

    def get_site_url(self, request, langcode='lg'):
        """returns a site urls form allready given keys"""
        return '%s%s/%s/%s/site/%s' % ('http://', request.META.get('HTTP_HOST'), \
        langcode, presettings.DYNAMIC_LINK_URL_BASE_COMPONENT, '-'.join(self.keys))











#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

VERSION = (0,5,0)
APPLICATION_NAME = "Dynamic Link"
VERSION_str = str(VERSION).strip('()').replace(',','.').replace(' ','')
VERSION_INFO = """
Version: %s
Modification date: 03.08.2011

Hints:


- 0.5.0     some changes - stable release
- 0.4.9.4   install_requires in setup.py modified
- 0.4.9.3   old data removed
- 0.4.9.2   dl_settings removed
- 0.4.9.1   Readme updatet
- 0.4.9     Testet with Django 1.2 u. 1.3
- 0.4.8.4   Some small changes
- 0.4.8.3   Some small changes
- 0.4.8     Runs now under Django 1.3
- 0.4.7   - Preesettings can now be overwrite in the presettings.py in the project
            folder, or directly in the global settings.py, or in a file
            called dl_settings.py in the same directory as manage.py.
            - django-dynamic-link now runs with python below version 2.6
- 0.4.6 -   changes in setup.py to avoid to install the example project.
- 0.4.5 -   Small adjustments at the example project.
- 0.4.4 -   1. Because of incomplete setup.py the last distributed package
            does't contained temples and language files.
            2. Readme updated.
            3. Example project added.
- 0.4.3 -   ImportError in admin.py solved.
- 0.4.2 -   setup.py createtd. Now listed in pypi.
- 0.4.1 -   Remove small bug with links created with the action menu.

TODO:
    - write more unittests
""" % (VERSION_str,)


from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

from dynamicLink import presettings

urlpatterns = patterns('',
    # Example:
    (r'^$', direct_to_template, {'template': 'home.html'}),

    # for django-dynamic-link. By default it catch url/serve/some-dynamic-link/
    (r'^\w+/%s/' % presettings.DYNAMIC_LINK_URL_BASE_COMPONENT, include('dynamicLink.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^admin/', include(admin.site.urls)),
)



#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

__author__ = "Andreas Fritz - sources.e-blue.eu"
__copyright__ = "Copyright (c) " + "28.08.2010" + " Andreas Fritz"
__licence__ = """New BSD Licence"""
__doc__ = """Dynamic donload links with timeout and maximum rate of ckicks.
The content will served by a stream. Path of the file on the system is covered
through the dynamicly generated pathname in the url.py """

import os
import presettings
from django.http import HttpResponse, HttpResponseNotFound
from django.shortcuts import render_to_response
from django.template import RequestContext
import mimetypes
from django.views.decorators.cache import cache_control
from django.utils.translation import ugettext_lazy as _
from models import Download, IsExpiredError


def expired(key):
    expired_objects = Download.objects.filter(active=False)
    # check expired objects
    for obj in expired_objects:
        if key == obj.link_key:
            return obj

def active(key):
    active_objects = Download.objects.filter(active=True)
    # check active objects
    for obj in active_objects:
        if key == obj.link_key:
            return obj

def error(request, text=_(u'Sorry, your request is mot available')):
    """returns the error page"""
    extra_context = {'message': text}
    template = 'dynamicLink/not_avallible.html'
    return render_to_response(template, extra_context, context_instance=RequestContext(request))

def site(request, offset):
    """process site requests"""
    offset = offset.split('-') # turn offset to keylist
    obj = {'actives':[], 'expired':[], 'notexist':[]}
    # Test if keys are valid
    for key in offset:
        if active(key):
            obj['actives'].append(active(key))
        elif expired(key):
            obj['expired'].append(expired(key))
        else:
            obj['notexist'].append(key)
    template = 'dynamicLink/provide.html'
    extra_context = {'basepath': presettings.DYNAMIC_LINK_URL_BASE_COMPONENT, 'downloads':obj}
    return render_to_response(template, extra_context, context_instance=RequestContext(request))

def fetch(request, offset):
    """process link requests. make desissions for every single download link"""
    if active(offset):
        return provide(request, offset)
    elif expired(offset):
        return error(request, presettings.TEXT_REQUEST_IS_EXPIRED)
    else:
        return error(request, presettings.TEXT_REQUEST_DOES_NOT_EXIST)

@cache_control(private=True)
def provide(request, key):
    """
    Return a download without the rael path to the served file. The content will
    served by a stream.

    The file will read in byte code to a socket wich will be used in the response
    object. Headers in the response will set for the specific served content
    referable to its type.
    """
    # Read database
    # This code runs only with python >= 2.6
#    stored_file_obj = Download.objects.get(link_key=key)
#    try:
#        filepath = stored_file_obj.get_path() # model method to validate and deliver path
#    except IsExpiredError as e: # works only with python 2.6 or later (for a solution of older versions see below!)
#        return error(request, e.value)

    # alternate code that also run in older python before 2.6
    stored_file_obj = Download.objects.get(link_key=key)
    try:
        filepath = stored_file_obj.get_path() # model method to validate and deliver path
    except IsExpiredError: # works with pyhton before 2.6
        return error(request)
    
    # make file path suitable for different installations
    delimiter = presettings.DYNAMIC_LINK_MEDIA.strip('/').split('/')[-1]
    # now we use the objects get_paht() method to be sure the object instance keep up to date.
    file_path = os.path.normpath(presettings.DYNAMIC_LINK_MEDIA + '/' + filepath.split(delimiter)[-1])

    # read file as binary
    try:
        fsocket = open(file_path, 'rb') # garbage collector will deal with not closed fsocket
    except IOError:
        stored_file_obj.active = False
        stored_file_obj.save() # only raise the following once
        return HttpResponseNotFound(unicode(_(u'File not found!'))) # admin will get informed by mail

#    # read file as binary
#    try:
#        f = open(file_path, 'rb')
#    except IOError:
#        stored_file_obj.active = False
#        stored_file_obj.save() # only raise the following once
#        return HttpResponseNotFound(unicode(_(u'File not found!'))) # admin will get informed by mail
#    fsocket = f.read()
#    f.close()

    # get file parameters
    file_name = os.path.basename(file_path)
    file_size = os.path.getsize(file_path)

    # specify mimetype and encoding
    auto_mimetype, auto_encoding = mimetypes.guess_type(file_path)
    if not auto_mimetype: # for unknown types use stream
        auto_mimetype = 'application/octet-stream'

    # response object
    response = HttpResponse(fsocket, mimetype=auto_mimetype) # object instance with mimetype and file
    # set headers in the response object
    # a list of headers: http://en.wikipedia.org/wiki/List_of_HTTP_header_fields
    # encode('utf-8') assuming you're running on a server with UTF-8 as the filesystem encoding.
    response['Content-Disposition'] = 'attachment; filename=%s' % file_name.encode('utf-8') # add correct filename
    if auto_encoding and auto_encoding is not 'gzip':
        # set encoding but exclude gzip from encoding headers
        # GZip uses zlib, but on its own zlib produces content that's improperly
        # encoded for a browser seeing 'gzip' as the content encoding.
        response['Content-Encoding'] = auto_encoding
    response['Content-Length'] = file_size # set response file size for the browsers progress bar
    
    return response



#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

__author__ = "Andreas Fritz - sources.e-blue.eu"
__copyright__ = "Copyright (c) " + "28.08.2010" + " Andreas Fritz"
__licence__ = """New BSD Licence"""
__doc__ = """Zur Zeit noch keine Dokumentation."""

from django.contrib import admin
from models import Download
from django.utils.translation import ugettext_lazy as _
import api
import presettings


class DownLinkAdmin(admin.ModelAdmin):
    def queryset(self, request):
        """catch the request object for list pages"""
        self.request = request
        return super(DownLinkAdmin, self).queryset(request)

    list_display = ('slug', 'active', 'file', 'valid', 'clicks', 'timestamp_creation', 'link')
    actions = ['make_link']
    search_fields = ['slug', 'file_path', 'timestamp_creation', 'link_key']
    list_per_page = 50
    fieldsets = (
                 (_(u'Link'), {
                 'fields': ('slug', 'file_path')
                 }),
                 (_(u'Aditional values'), {
                 'classes': ('collapse',),
                 'fields': ('active', 'current_clicks', 'timeout_hours', 'max_clicks')
                 }),
                 )

    def valid(self, obj):
        """Shows timestamp expired or active time"""
        diff = unicode(obj.get_timout_time()).split('.')[0]
        if obj.timeout_time():
            if obj.active:
                # set active to false
                obj.active = False
                obj.save()
            return '<span style="color: #FF7F00; ">%s</span>:<br/> ' \
            % (unicode(_(u'timeout'))) + diff
        else:
            return diff
    valid.allow_tags = True
    valid.short_description = _(u'valid')

    def file(self, obj):
        """Shows truncated filename on platform indepentend length."""
        return unicode(obj.file_path).split(presettings.DYNAMIC_LINK_MEDIA)[-1]
    file.allow_tags = True
    file.short_description = _(u'file')

    def clicks(self, obj):
        """Shows current and max allowed clicks in the list display"""
        txt = '%s %s %s' % (obj.current_clicks, unicode(_(u'from')), obj.max_clicks)
        if obj.timeout_clicks():
            if obj.active == True:
                # set active to false
                obj.active = False
                obj.save()
            return '<span style="color: #FF7F00; ">%s</span><br/>%s' \
            % (unicode(_('max clicks reached')), txt)
        elif obj.max_clicks == 0:
            return '%s %s <span style="color: #FF7F00; ">%s</span>' \
            % (obj.current_clicks, unicode(_(u'from')), unicode(_(u'unlimited')))
        else:
            return txt
    clicks.allow_tags = True
    clicks.short_description = _(u'clicks')

    def link(self, obj):
        """Generate site and download url from link object"""

        # download site with link
        siteurl = api.DownloadSiteUrl([obj.link_key])
        sitelink = siteurl.get_site_url(self.request)
        sitelink = u'<span style="color: #FF7F00; ">%s:</span> \
        <a target="new" href="%s/">%s/</a><br/>' % (unicode(_(u'Site')), sitelink, sitelink)

        # direct accessable link
        filelink = api.file_link_url(self.request, obj)
        filelink = '<span style="color: #FF7F00; ">%s:</span> %s' % (unicode(_(u'File')), filelink)

        return sitelink + filelink
    link.allow_tags = True
    link.short_description = _(u'link')

    def make_link(modeladmin, request, queryset):
        """Action method. Make site url from many singles objects."""
        li = []
        for obj in queryset:
            li.append(obj.link_key)
        siteurl = api.DownloadSiteUrl(li)
        sitelink = siteurl.get_site_url(request)
        # roesponse
        from django.http import HttpResponse
        return HttpResponse('<a target="new" href="%s/">%s/</a><br/>' % (sitelink, sitelink))
    make_link.short_description = _("Make from selected a download site link")

admin.site.register(Download, DownLinkAdmin)


#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

from setuptools import setup, find_packages
from dynamicLink import version
import os
import sys

# Read the version from a project file
VERSION = version.VERSION_str

# Get description from Readme file
long_description = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# Build a list with requirements of the app
REQUIRES = ['django']
if sys.version_info < (2, 4):
    REQUIRES.append('python >= 2.4')

setup(name='django-dynamic-link',
        version=VERSION,
        description='A django file streaming application',
        long_description=long_description,
        author='Andreas Fritz',
        author_email='djangp-dynamic-link@bitzeit.eu',
        url='http://www.sources.e-blue.eu/de/pages/django/',
        download_url='https://bitbucket.org/catapela/django-dynamic-link/downloads',
        license='BSD',
        packages=find_packages(exclude=['example',]),
        include_package_data=True,
        keywords="django file streaming dynamic links serve",
        classifiers=[
              'Development Status :: 4 - Beta',
              'Framework :: Django',
              'License :: OSI Approved :: BSD License',
              'Operating System :: OS Independent',
              'Programming Language :: Python',
              'Environment :: Console',
              'Natural Language :: English',
              'Natural Language :: German',
              'Intended Audience :: Developers',
              'Intended Audience :: Information Technology',
              'Topic :: Internet',
              'Topic :: Utilities',
              ],
        install_requires = REQUIRES,
        zip_safe=False,
        )



# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8

from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('', 
                       (url(r'^site/([\w-]*)/$', 'dynamicLink.views.site')),
                       (url(r'^link/(\w{1,100})/.*$', 'dynamicLink.views.fetch')),
                       (url(r'^link/(\w{1,100})$', 'dynamicLink.views.fetch')), # without prefix
                       )
